"""
Module containing default tolerances for plotting outputs

All unique {keys:tolerances} for plotting outputs.
Each entry has a unique key and value = {'tolerance':value, 'unit':Unit.unit}
where Unit is an enum class.  If a quantity does not have an associated unit,
unit = null.

Keys were generated by using the tolerance.py script.
See that for command line details.

Tolerances in default_tolerances cover files:
 * RHO3D.xml
 * VCL3D.xml
 * VXC3D.xml
 * WF3D.xml
 * ELF3D.xml
 * EF3D.xml
"""
from ..tol_classes import DefaultTolerances, Tol, TolWithMessage


default = DefaultTolerances(integer=Tol(0),
                            float=Tol(1.e-8),
                            str=Tol('')
                            )

plot3d = {'originrs': default.float,
          'latexunit': default.integer,
          'gridticks': default.integer,
          'label': default.str,
          'endpoint': default.float,
          'graceunit': default.str,
          'origin': default.float,
          'title': default.str,
          'endpointrs': default.float,
          'data': default.float,
          'index': default.integer,
          'n': default.integer,
          'const': default.str,
          'name': default.str,
          'delta': default.float
          }

plotting_tolerances = {'RHO3D.xml': plot3d,
                       'VCL3D.xml': plot3d,
                       'VXC3D.xml': plot3d,
                       'WF3D.xml': plot3d,
                       'ELF3D.xml': plot3d,
                       'EF3D.xml': plot3d}