"""
Module containing default tolerances for wannier output.

All unique {keys:tolerances} for wannier outputs.
Each entry has a unique key and value = {'tolerance':value, 'unit':Unit.unit}
where Unit is an enum class.  If a quantity does not have an associated unit,
'unit = None.

Keys were generated by using the tolerance.py script. See that for command line
details.

Tolerances in default_tolerances cover files:
  * TDOS_WANNIER.OUT
  * WANNIER_INFO.OUT
"""
from excitingtools import Unit

from ..tol_classes import DefaultTolerances, Tol, TolWithMessage

default = DefaultTolerances(integer=Tol(0),
                            float=Tol(1.e-8),
                            length=Tol(1.e-8, Unit.bohr),
                            energy=Tol(1.e-6, Unit.hartree),
                            dos=Tol(1.e-8, Unit.inv_hartree)
                            )

# TDOS_WANNIER.OUT file
tdos_wannier = {'energy': default.energy,
                'dos': default.dos
                }

# WANNIER_INFO.OUT file
wannier_info = {'n_wannier': default.integer,
                'localisation_center': default.length,
                'Omega': default.float,
                'Omega_I': default.float,
                'Omega_D': default.float,
                'Omega_OD': default.float
                }

# Single dictionary for all tested Wannier outputs, for dumping to JSON
wannier_tolerances = {'TDOS_WANNIER.OUT': tdos_wannier, 
                      'WANNIER_INFO.OUT': wannier_info}
