# Tutorial scripts

This folder contains the (legacy) exciting scripts, which were mainly used in the tutorials. They include _Python 2_ and (partially) _3_ scripts, _bash_ scripts, _gnuplot_ inputs, and some other, unidentified, files.

Below we list these files, including in which tutorials they are used and sketch their functionality.

In future versions of the code, they will be replaced by Python 3 modules, which can be invoked via the Python interpreter, or (TODO) directly as a command line tool.

## Script listing

### SETUP

#### <span style="color:#15317E">SETUP-excitingroot.sh</span>

Replace placeholder "$EXCITINGROOT" in **input.xml** files by actual path.

#### New version:
Located at `tools/excitingscripts/excitingscripts/setup/excitingroot.py`. Call as:

```bash
python3 -m excitingscripts.setup.excitingroot
```

#### Used in:
All tutorials
</br></br>
#### <span style="color:#15317E">SETUP-convergence-test.py</span>

Generate input files with different values of the main computational parameters.
#### New version:
Located at `tools/excitingscripts/excitingscripts/setup/convergence_test.py`. Call as:

```bash
python3 -m excitingscripts.setup.convergence_test k_i k_f rgkmax_i rgkmax_f
```
Where <code>k_i</code> and <code>k_f</code> are the initial and final k-values for defining the 
<code><span style="color:green">groundstate</span></code> attribute 
<code><span style="color:MediumBlue">ngridk</span></code>, and <code>rgkmax_i</code> and <code>rgkmax_f</code> the 
initial and final values for the 
<code><span style="color:green">groundstate</span></code> attribute 
<code><span style="color:MediumBlue">rgkmax</span></code>.
</br></br>
#### Used in:
* Simple Convergence Tests
</br></br>
#### <span style="color:#15317E">SETUP-dft0.5.py</span>

Generate a set of input files varying the attribute <code><span style="color:mediumblue">cut</span></code>.

#### Used in:
* DFT-1/2
</br></br>
#### <span style="color:#15317E">SETUP-diamond-phonon-g.py</span>

Generate structures with displaced atoms.

#### Used in:
* Phonons at Γ in Diamond-Structure Crystals
</br></br>
#### <span style="color:#15317E">SETUP-diamond-phonon-x.py</span>

Generate structures with displaced atoms.

#### Used in:
* Phonons at X in Diamond-Structure Crystals
</br></br>
#### <span style="color:#15317E">SETUP-elastic-strain.py</span>

Generate strained structures.

#### Used in:
* Energy vs. Strain Calculations
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">SETUP-graphene-along-c.py</span>

Generate graphene structures in which an atom is displaced along the c-axis.

#### Used in:
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">SETUP-interlayer-distance.py</span>

Generate structures with different interlayer distances.
#### New version:
Located at `tools/excitingscripts/excitingscripts/setup/interlayer_distance.py`. Call as:

```bash
python3 -m excitingscripts.setup.interlayer_distance dmin dmax nr_displ dinfty
```
Where <code>dmin</code> and <code>dmax</code> are the minimum and maximum values for the interlayer distance, 
<code>nr_displ</code> is the number of distances in the interval [<code>dmin</code>, <code>dmin</code>] and
<code>dinfty</code> is the interlayer distance at infinity.

#### Used in:
* Van-der-Waals Corrections
</br></br>
#### <span style="color:#15317E">SETUP-planar.py</span>

Set calculations for two-dimensional materials.

#### Used in:
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">SETUP-volume-optimization.py</span>

Generate structures at different volumes.
#### New version:
Located at `tools/excitingscripts/excitingscripts/setup/volume_optimization.py`. Call as:

```bash
python3 -m excitingscripts.setup.volume_optimization nr_vol
```
Where <code>nr_vol</code> is the number of volume values for which structures are generated by varying the lattice
constant.
#### Used in:
* Volume Optimization for Cubic Systems
* Exchange-Correlation Functionals
</br></br>
#### <span style="color:#15317E">setup.band_structure.py</span>

Add properties element with bandstructure element to given input file by getting the band path from the lattice cell.
**Note**: If a properties tree already exists, it will be overwritten!

Located at `tools/excitingscripts/excitingscripts/setup/band_structure.py`. Call as:

```bash
python3 -m excitingscripts.setup.band_structure
```
</br></br>
### EXECUTE

#### <span style="color:#15317E">EXECUTE-convergence-test.sh</span>

Run a series of **exciting** calculations with different values of the main computational parameters.
#### New version:
Located at `tools/excitingscripts/excitingscripts/execute/convergence_test.py`. Call as:

```bash
python3 -m excitingscripts.execute.convergence_test k_i k_f rgkmax_i rgkmax_f
```
Where <code>k_i</code> and <code>k_f</code> are the initial and final k-values for defining the 
<code><span style="color:green">groundstate</span></code> attribute 
<code><span style="color:MediumBlue">ngridk</span></code>, and <code>rgkmax_i</code> and <code>rgkmax_f</code> the 
initial and final values for the 
<code><span style="color:green">groundstate</span></code> attribute 
<code><span style="color:MediumBlue">rgkmax</span></code>.
</br></br>
#### Used in:
* Simple Convergence Tests
</br></br>
#### <span style="color:#15317E">EXECUTE-diamond-phonon-g.sh</span>

Run a series of **exciting** calculations (for example using input files created by SETUP-diamond-phonon-g.py).
#### Used in:
* Phonons at Γ in Diamond-Structure Crystals
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">EXECUTE-diamond-phonon-x.sh</span>

Run a series of **exciting** calculations (for example using input files created by SETUP-diamond-phonon-x.py).
#### Used in:
* Phonons at X in Diamond-Structure Crystals
</br></br>
#### <span style="color:#15317E">EXECUTE-elastic-strain.sh</span>

Run a series of **exciting** calculations (for example using input files created by SETUP-elastic-strain.py).
#### Used in:
* Energy vs. Strain Calculations
* Graphene: from the Ground State to Excitations
* Van-der-Waals Correlations
* DFT-1/2
</br></br>
#### <span style="color:#15317E">EXECUTE-planarAverage.py</span>

Extract planar-averaged electrostatic potential in a given direction.
#### Used in:
* Dipole Correction for Surface Calculations
</br></br>
#### <span style="color:#15317E">EXECUTE-single.sh</span>

Run a single **exciting** calculation.
#### New version:
Located at `tools/excitingscripts/excitingscripts/execute/single.py`. Call as:

```bash
python3 -m excitingscripts.execute.single -r rundir
```
Where <code>rundir</code> is an optional parameter which specifies the running directory.
If <code>rundir</code> is not specified, the calculation will run in the directory where the
script is called.
#### Used in:
* How to Visualize Kohn-Sham States
* Spin-Polarized Calculations for bcc Fe
* Hybrid-Functional Calculations
* Exact Exchange Calculations
* DFT-1/2
* Simple Examples of Structure Optimization
* How to Run Calculations for Simple Molecules
* Excited States from TDDFT
* Real-Time TDDFT
* q-dependent TDDFT
* Simulating Pump-Probe Spectroscopy
* Studying Higher-Harmonic Generation using RT-TDDFT
* Graphene: from the Ground State to Excitations
* Calculation of STM Spectra
* Magneto-Optical Kerr Effect
</br></br>

#### <span style="color:#15317E">EXECUTE-volume-optimization.sh</span>

Run a series of **exciting** calculations (for example using input files created by SETUP-volume-optimization.py).
#### New version:
Located at `tools/excitingscripts/excitingscripts/execute/volume_optimization.py`. Call as:

```bash
python3 -m excitingscripts.execute.volume_optimization nr_vol
```
Where <code>nr_vol</code> is the number of volume values for which structures are generated by varying the lattice
constant.
#### Used in:
* Volume Optimization for Cubic Systems
* Exchange-Correlation Functionals
</br></br>
### CHECKFIT

#### <span style="color:#15317E">CHECKFIT-energy-vs-displacement.py</span>

Extract the derivatives at zero displacement of the energy-vs-displacement curves.
#### Used in:
* Phonons at Γ in Diamond-Structure Crystals
* Phonons at X in Diamond-Structure Crystals
</br></br>
#### <span style="color:#15317E">CHECKFIT-energy-vs-strain.py</span>

Extract the derivatives at zero strain of the energy-vs-strain curves.
#### Used in:
* Energy vs. Strain Calculations
</br></br>
#### <span style="color:#15317E">CHECKFIT-energy-vs-volume.py</span>

Fit energy-vs-volume curves.
#### Used in:
* Volume Optimization for Cubic Systems
</br></br>
#### <span style="color:#15317E">CHECKFIT-force-vs-displacement.py</span>

Extract the derivatives at zero displacement of the force-vs-displacement curves.
#### Used in:
* Phonons at Γ in Diamond-Structure Crystals
</br></br>

### STRESS

#### <span style="color:#15317E">STRESS-exciting-analyze.py</span>

Fit the energy-vs-strain and CVe-vs-strain curves.
#### Used in:
* How to Calculate the Stress Tensor
</br></br>
#### <span style="color:#15317E">STRESS-exciting-clean.sh</span>

Clean unnecessary files.
#### Used in:
* How to Calculate the Stress Tensor
</br></br>
#### <span style="color:#15317E">STRESS-exciting-result.py</span>

Calculate the stress components.
#### Used in:
* How to Calculate the Stress Tensor
</br></br>
#### <span style="color:#15317E">STRESS-exciting-setup.py</span>

Generate structures at different strains.
#### Used in:
* How to Calculate the Stress Tensor
</br></br>
#### <span style="color:#15317E">STRESS-exciting-submit.sh</span>

Run a series of **exciting** calculations.
#### Used in:
* How to Calculate the Stress Tensor
</br></br>

### OPTIMIZE

#### <span style="color:#15317E">OPTIMIZE-analyze.py</span>

Fit the energy-vs-volume and energy-vs-strain curves. This script is called by the **OPTIMIZE-lattice.py** script.
#### Used in:
* General Lattice Optimization
</br></br>
#### <span style="color:#15317E">OPTIMIZE-lattice.py</span>

Calls the **OPTIMIZE-setup.py** and **OPTIMIZE-analyze.py** scripts.
#### Used in:
* General Lattice Optimization
</br></br>
#### <span style="color:#15317E">OPTIMIZE-setup.py</span>

Generate structures at different volume/strains. This script is used within the **OPTIMIZE-lattice.py** script.
#### Used in:
* General Lattice Optimization
</br></br>
#### <span style="color:#15317E">OPTIMIZE-submit.sh</span>

Run a series of **exciting** calculations.
#### Used in:
* General Lattice Optimization
</br></br>

### PLOT

#### <span style="color:#15317E">PLOT-3d-convergence.gnu</span>

Visualize convergence results.
#### New version:
Located at `tools/excitingscripts/excitingscripts/plot/convergence.py`. Call as:

```bash
python3 -m excitingscripts.plot.convergence rk
```
#### Used in:
* Simple Convergence Tests
</br></br>
#### <span style="color:#15317E">PLOT-atomforce.py</span>

Visualize force-vs-displacement curves.
#### Used in:
* Phonons at Γ in Diamond-Structure Crystals
</br></br>
#### <span style="color:#15317E">PLOT-band-structure.py</span>

Visualize band-structure.
#### New version:
Located at `tools/excitingscripts/excitingscripts/plot/band_structure.py`. Call as:

```bash
python3 -m excitingscripts.plot.band_structure
```

#### Used in:
* Electronic Band-Structure and Density of States
* Spin-polarized calculations for bcc Fe
* Spin-orbit coupling
* Hybrid-Functional Calculations
* Exact-Exchange Calculations
* Wannier Functions for Interpolation in Reciprocal Space
* Phonon Properties of Diamond-Structure Crystals
* Electronic Band Structure from GW
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">PLOT-bbirch.py</span>

Fit bulk-modulus-vs-volume curves using the Birch-Murnaghan equation of state (**BM-EoS**).
#### Used in:
* Volume Optimization for Cubic Systems
</br></br>
#### <span style="color:#15317E">PLOT-birch.py</span>

Fit energy-vs-volume curves using the Birch-Murnaghan equation of state (**BM-EoS**).
#### Used in:
* Volume Optimization for Cubic Systems
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">PLOT-pbirch.py</span>

Fit pressure-vs-volume curves using the Birch-Murnaghan equation of state (**BM-EoS**).
#### Used in:
* Volume Optimization for Cubic Systems
</br></br>
#### <span style="color:#15317E">PLOT-newbirch.py</span>

Fit energy-vs-volume curves using the Birch-Murnaghan equation of state (**BM-EoS**) in polynomial form.
#### New version:
Located at `tools/excitingscripts/excitingscripts/plot/newbirch.py`. Call as:

```bash
python3 -m excitingscripts.plot.newbirch
```
#### Used in:
* Volume Optimization for Cubic Systems
* Exchange-Correlation Functionals.
</br></br>
#### <span style="color:#15317E">PLOT-centerofmass.py</span>

Visualize the cartesian components of the position of the center of mass during relaxation.
#### Used in:
* How to Run Calculations for Simple Molecules
</br></br>
#### <span style="color:#15317E">PLOT-checkderiv.py</span>

Fit the derivatives of the energy-vs-displacement curves at zero displacement.
#### Used in:
* Phonons at Γ in Diamond-Structure Crystals
* Phonons at X in Diamond-Structure Crystals
* Energy vs. Strain Calculations
</br></br>
#### <span style="color:#15317E">PLOT-convergence.py</span>

Visualize convergence results.
#### New version:
Located at `tools/excitingscripts/excitingscripts/plot/convergence.py`. Call as:

```bash
python3 -m excitingscripts.plot.convergence plot_mode
```
Where <code>plot_mode</code> is either <code>k</code> for plotting energy curves with varying values of the <code><span style="color:green">groundstate</span></code> attribute 
<code><span style="color:MediumBlue">ngridk</span></code>, <code>r</code> for varying values of the 
<code><span style="color:green">groundstate</span></code> attribute 
<code><span style="color:MediumBlue">rgkmax</span></code> or 
<code>rk</code> for a 3D plot with varying values of both attributes.
#### Used in:
* Simple Convergence Tests
</br></br>
#### <span style="color:#15317E">PLOT-compare-vdW.py</span>

Visualize multiple energy-vs-distance curves.
#### Used in:
* Van-der-Waals Corrections
</br></br>
#### <span style="color:#15317E">PLOT-dos.py</span>

Visualize desisty of states.
#### New version:
Located at `tools/excitingscripts/excitingscripts/plot/dos.py`. Call as:

```bash
python3 -m excitingscripts.plot.dos
```
#### Used in:
* Electronic Band-Structure and Density of States
* Spin-polarized calculations for bcc Fe
* Wannier Functions for Interpolation in Reciprocal Space
* Phonon Properties of Diamond-Structure Crystals
* Electronic Band Structure from GW
* Graphene: from the Ground State to Excitations
</br></br>
#### <span style="color:#15317E">PLOT-energy.py</span>

Visualize energy-vs-strain curves.
#### Used in:
* Van-der-Walls Corrections.
* Phonons at Γ in Diamond-Structure Crystals
* Phonons at X in Diamond-Structure Crystals
* Energy vs. Strain Calculations
</br></br>
#### <span style="color:#15317E">PLOT-exciton-weights-XX.py</span>
PLot the excitonic weights along a band structure path, where XX stands for LiF or BN.
#### Used in:
* Exciton Analysis and Visualization
</br></br>
#### <span style="color:#15317E">PLOT-files.py</span>

Visualize data included in different files and different directories..
#### Used in:
* How to Visualize Kohn-Sham States
* Dipole Correction for Surface Calculations
* DFT-1/2
* Phonon Properties of Diamond-Structure Crystals
* Excited States from BSE
* Exciton Analysis and Visualization 
* X-Ray Absorption Spectra using BSE
* X-Ray Emission Spectra
* q-Dependent BSE Calculations
* Graphene: from the Ground State to Excitations
* Transport Properties Using the Boltzmann Equation
</br></br>
#### <span style="color:#15317E">PLOT-loss-function.py</span>

Visualize the loss function.
#### Used in:
* Excited States from TDDFT
* q-Dependent TDDFT
</br></br>
#### <span style="color:#15317E">PLOT-maxforce.py</span>

Visualize the maximum amplitude of the force on the atoms during relaxation.
#### Used in:
* Simple Examples of Structure Optimization
* How to Run Calculations for Simple Molecules
* Energy vs. Strain Calculations
</br></br>
#### <span style="color:#15317E">PLOT-MOKE-spectrum.py</span>

Visualize the MOKE spectrum.
#### Used in:
* Magneto-Optical Kerr Effect (MOKE)
</br></br>
#### <span style="color:#15317E">PLOT-multitask.py</span>

Make plots from output files of RT-TDDFT.
#### Used in:
* Real-Time TDDFT
* Simulating Pump-Probe Spectroscopy
* Studying Higher-Harmonic Generation using RT-TDDFT
</br></br>
#### <span style="color:#15317E">PLOT-optimized-geometry.py</span>

Visualization tool for relaxed coordinates of atoms in the unit cell.
#### Used in:
* Graphene: from the Ground State to Excitations
* Energy vs. Strain Calculations
</br></br>
#### <span style="color:#15317E">PLOT-phonon-anim.py</span>

Produce coordinate files of animated phonon modes.
#### Used in:
* How to Visualize Phonons
</br></br>
#### <span style="color:#15317E">PLOT-poly.py</span>

Fit energy-vs-volume curves using a polynomial fit in the volume.
#### Used in:
* Volume Optimization for Cubic Systems
</br></br>
#### <span style="color:#15317E">PLOT-relaxdistance.py</span>

Follow relative atomic coordinates of atoms during the relaxation process.
#### Used in:
* Simple Examples of Structure Optimization
* How to Run Calculations for Simple Molecules
</br></br>
#### <span style="color:#15317E">PLOT-SHG.py</span>

Visualize second-harmonic generation (**SHG**) spectrum.
#### Used in:
* Second-Harmonic Generation
</br></br>
#### <span style="color:#15317E">PLOT-spectra.py</span>

Generate plots of the dielectric function.
#### Used in:
* Many-Body Kernels for TDDFT Calculations
</br></br>
#### <span style="color:#15317E">PLOT-spintext.py</span>

Produce plot of the spin texture.
#### Used in:
* Spin-texture calculation
</br></br>
#### <span style="color:#15317E">PLOT-status.py</span>

Visualize **RMS** deviations of the **SCF** potential as a function of the iteration number during the **SCF** loop.
#### Used in:
* Simple Examples of Structure Optimization
* How to Run Calculations for Simple Molecules
* Phonons at Γ in Diamond-Structure Crystals
* Phonons at X in Diamond-Structure Crystals
* Energy vs. Strain Calculations
</br></br>
#### <span style="color:#15317E">PLOT-stm.py</span>

Visualize Scanning Tunneling Microscope (**STM**) images.
#### Used in:
* Calculation of STM Spectra
</br></br>
#### <span style="color:#15317E">PLOT-torque.py</span>

Visualize the total torque during relaxation.
#### Used in:
* How to Run Calculations for Simple Molecules
</br></br>
#### <span style="color:#15317E">PLOT-totalmoment.py</span>

Visualize the volume dependence of total magnetic moment.
#### Referenced in:
* Spin-Polarized Calculations for bcc Fe
</br></br>
#### <span style="color:#15317E">PLOT-vinet.py</span>

Fit energy-vs-volume curves using the Vinet equation of state.
#### Used in:
* Volume Optimization for Cubic Systems
</br></br>
#### <span style="color:#15317E">PLOT-volumecurves.py</span>

Fit energy-vs-volume curves.
#### New version:
Located at `tools/excitingscripts/excitingscripts/plot/volumecurves.py`. Call as:

```bash
python3 -m excitingscripts.plot.volumecurves -r dir1 dir2 dir3
```
Where <code>dir1</code>, <code>dir2</code>, <code>dir3</code> take the place of the names of the  directories where
exciting calculations have been performed. The script can be used for any number of directories.

#### Used in:
* Exchange-Correlation Functionals
</br></br>
#### <span style="color:#15317E">PLOT-wannier-function.sh</span>

Plot Wannier Functions.
#### Used in:
* Wannier Functions for interpolation in reciprocal space
</br></br>

### Other Scripts

#### <span style="color:#15317E">COMPARE-transition-energies.py</span>

Determine the transition energies for the transitions Γ→Γ and Γ→X for given directories in which exciting
calculations have been performed.
#### New version:
Located at `tools/excitingscripts/excitingscripts/compare_transition_energies.py`. Call as:

```bash
python3 -m excitingscripts.compare_transition_energies -r dir1 dir2 dir3
```
Where <code>dir1</code>, <code>dir2</code>, <code>dir3</code> take the place of the names of the  directories where
exciting calculations have been performed, which need to be specified in order to calculate the transition energies.
The script can be used for any number of directories.
#### Used in:
* Hybrid-Functional Calculations
</br></br>

#### <span style="color:#15317E">CONVERT-xml2xsf.sh</span>

Convert **xml** files to **xsf**.
#### Used in:
* How to visualize Kohn-Sham States
* How to Run Calculations for Simple Molecules
</br></br>
#### <span style="color:#15317E">Grace.par</span>

File containing **xmgrace** parameters needed by visualization tools.
#### Used in:
* How to Calculate the Stress Tensor
</br></br>
#### <span style="color:#15317E">exciting2sgroup.py</span>

Convert the **exciting** input to an **sgroup** input file.
#### Used in:
* How to Calculate the Stress Tensor
* General Lattice Optimization
</br></br>
#### <span style="color:#15317E">FREE-fromdos.py</span>

Calculate thermodynamic properties.
#### Used in:
* Phonon Properties of Diamond-Structure Crystals
</br></br>
#### <span style="color:#15317E">LATTICE-parameters.sh</span>

Extract the values of the lattice parameters out of any **exciting** input file.
#### Used in:
* How to Calculate the Stress Tensor
* General Lattice Optimization
